# -*- coding: utf-8 -*-

"""Main module."""

# TODO: Floating IP Addresses

import os
import json

import git
import boto3
from slugify import slugify

from toolz.functoolz import compose
from troposphere import Template

from stacks.ec2_instance import ec2_instance
from stacks.security_group import security_group
from stacks.elastic_ip import elastic_ip
from stacks.utils import get_parameters, create_hash

from errors import ParameterError
from aws import Client
from utils import output_success_results, write_state, setup_project
from repo import (
    record_create_attempt,
    record_successful_create,
    record_successful_update,
    rollback
)


class Stack():
    # user Settings
    repo = None
    repo_dir = None
    project = None
    project_dir = None

    def __init__(self, project, repo, profile="hlk_superadmin", test=False):
        self.template = Template()

        self.test = test
        self.project = slugify(project)
        self.repo_dir = os.path.expanduser('~/.hlk_infrastructure')
        self.project_dir = os.path.join(self.repo_dir, slugify(project))

        self.client = Client(profile=profile, test=test)
        self.repo = setup_project(
            repo,
            self.project,
            self.repo_dir,
            self.project_dir,
            test=test
        )

    # Create an EC2 Instance
    def ec2(
        self,
        meta,
        keyname='developer',
        security_groups=["default"],
        type="t2.nano",
        block_devices=[{
            'name': '/dev/sdf',
            'dir': '/var/www',
            'size': 8
        }],
        commands=[],
        count=1
    ):
        self.templates = ec2_instance(
            self.template,
            keyname,
            security_groups,
            type,
            block_devices,
            commands,
            meta,
            count,
            self.client.session
        )

    def security_group(
        self,
        name,
        ports={
            '80': '0.0.0.0/0',
            '443': '0.0.0.0/0',
            '22': '0.0.0.0/0',
        },
        allow_from=None
    ):
        self.templates = security_group(
            self.template,
            name,
            ports,
            allow_from
        )

    def elastic_ip(self, instance):
        self.templates = elastic_ip(
            self.template,
            instance
        )

    # Build Infrastructure and Save State
    def build(self, feature, tags={}):
        # Get generated state, hash, and check if this projec exists
        state = self.template
        json_state = state.to_json()
        hash = create_hash(json_state)
        state_exists = os.path.isfile(
            os.path.join(self.project_dir, 'state.json')
        )

        # If no prior state exists
        if not state_exists:
            # No State should exist remotely (describe stacks) -- Error if it does
            # TODO: Check existing state in AWS

            # Write our new state file for this project
            write_state(self.project_dir, state, test=self.test)
            record_create_attempt(self.repo, hash, self.project, test=self.test)

            create = self.client.create_stack(self.project, state)
            if create:
                output_success_results(create)
                record_successful_create(
                    self.repo,
                    hash,
                    feature,
                    self.project,
                    test=self.test
                )
                return create
            else:
                rollback(self.repo, hash, test=self.test)

        # Else we have prior state
        else:
            write_state(self.project_dir, state)
            if len(self.repo.untracked_files) > 0 or self.repo.is_dirty():
                record_create_attempt(
                    self.repo,
                    hash,
                    self.project,
                    test=self.test
                )
                update = self.client.update_stack(self.project, state)
                if update:
                    output_success_results(update)
                    record_successful_update(
                        self.repo,
                        hash,
                        feature,
                        self.project,
                        test=self.test
                    )
                    return update
                else:
                    rollback(self.repo, hash, test=self.test)

            else:
                # No Changes to process
                # TODO: Check to ensure state matches what is in AWS
                print("Project '%s' state has not changed..." % self.project)
